<?php return array(
    'root' => array(
        'name' => '__root__',
        'pretty_version' => 'dev-main',
        'version' => 'dev-main',
        'reference' => '2ad8f7990f64a48c28df70c245c05ceb4a018273',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        '__root__' => array(
            'pretty_version' => 'dev-main',
            'version' => 'dev-main',
            'reference' => '2ad8f7990f64a48c28df70c245c05ceb4a018273',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ergebnis/phpstan-rules' => array(
            'pretty_version' => '2.13.1',
            'version' => '2.13.1.0',
            'reference' => 'f69db86b98595c34fc1f61c89fe3b380141aa519',
            'type' => 'phpstan-extension',
            'install_path' => __DIR__ . '/../ergebnis/phpstan-rules',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpstan/extension-installer' => array(
            'pretty_version' => '1.4.3',
            'version' => '1.4.3.0',
            'reference' => '85e90b3942d06b2326fba0403ec24fe912372936',
            'type' => 'composer-plugin',
            'install_path' => __DIR__ . '/../phpstan/extension-installer',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpstan/phpstan' => array(
            'pretty_version' => '2.2.8',
            'version' => '2.2.8.0',
            'reference' => 'e285254e60f33c21902efef4a926ca0987c06804',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpstan/phpstan',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'phpstan/phpstan-strict-rules' => array(
            'pretty_version' => '2.0.12',
            'version' => '2.0.12.0',
            'reference' => '2bc5ae19ae965663b62ac907ee6342c3903ec93b',
            'type' => 'phpstan-extension',
            'install_path' => __DIR__ . '/../phpstan/phpstan-strict-rules',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'tomasvotruba/type-coverage' => array(
            'pretty_version' => '2.3.0',
            'version' => '2.3.0.0',
            'reference' => '7ba48c1bf3bbb7ffe99caacc7017bae5ee72d644',
            'type' => 'phpstan-extension',
            'install_path' => __DIR__ . '/../tomasvotruba/type-coverage',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
        'webmozart/assert' => array(
            'pretty_version' => '2.4.1',
            'version' => '2.4.1.0',
            'reference' => '2ccb7c2e821038c03a3e6e1700c570c158c55f70',
            'type' => 'library',
            'install_path' => __DIR__ . '/../webmozart/assert',
            'aliases' => array(),
            'dev_requirement' => true,
        ),
    ),
);
