<?php

declare(strict_types=1);

namespace Rector\TypePerfect\Tests\Rules\NarrowPublicClassMethodParamTypeRule\Fixture;

final class SkipProperlyFilledParamType
{
    public function callMeTwice(int $number)
    {
    }
}
