<?php

// autoload_static.php @generated by Composer

namespace Composer\Autoload;

class ComposerStaticInitf9e7218f71d5874b5632927df4f72bd7
{
    public static $files = array (
        '9b38cf48e83f5d8f60375221cd213eee' => __DIR__ . '/..' . '/phpstan/phpstan/bootstrap.php',
    );

    public static $prefixLengthsPsr4 = array (
        'W' =>
        array (
            'Webmozart\\Assert\\' => 17,
        ),
        'T' =>
        array (
            'TomasVotruba\\TypeCoverage\\' => 26,
        ),
        'R' =>
        array (
            'Rector\\TypePerfect\\' => 19,
        ),
        'P' =>
        array (
            'PHPStan\\ExtensionInstaller\\' => 27,
            'PHPStan\\' => 8,
        ),
        'E' =>
        array (
            'Ergebnis\\PHPStan\\Rules\\' => 23,
        ),
    );

    public static $prefixDirsPsr4 = array (
        'Webmozart\\Assert\\' =>
        array (
            0 => __DIR__ . '/..' . '/webmozart/assert/src',
        ),
        'TomasVotruba\\TypeCoverage\\' =>
        array (
            0 => __DIR__ . '/..' . '/tomasvotruba/type-coverage/src',
        ),
        'Rector\\TypePerfect\\' =>
        array (
            0 => __DIR__ . '/..' . '/tomasvotruba/type-coverage/packages/type-perfect/src',
        ),
        'PHPStan\\ExtensionInstaller\\' =>
        array (
            0 => __DIR__ . '/..' . '/phpstan/extension-installer/src',
        ),
        'PHPStan\\' =>
        array (
            0 => __DIR__ . '/..' . '/phpstan/phpstan-strict-rules/src',
        ),
        'Ergebnis\\PHPStan\\Rules\\' =>
        array (
            0 => __DIR__ . '/..' . '/ergebnis/phpstan-rules/src',
        ),
    );

    public static $classMap = array (
        'Composer\\InstalledVersions' => __DIR__ . '/..' . '/composer/InstalledVersions.php',
    );

    public static function getInitializer(ClassLoader $loader)
    {
        return \Closure::bind(function () use ($loader) {
            $loader->prefixLengthsPsr4 = ComposerStaticInitf9e7218f71d5874b5632927df4f72bd7::$prefixLengthsPsr4;
            $loader->prefixDirsPsr4 = ComposerStaticInitf9e7218f71d5874b5632927df4f72bd7::$prefixDirsPsr4;
            $loader->classMap = ComposerStaticInitf9e7218f71d5874b5632927df4f72bd7::$classMap;

        }, null, ClassLoader::class);
    }
}
