<?php

declare(strict_types=1);

namespace Rector\TypePerfect\Tests\Rules\NarrowPublicClassMethodParamTypeRule\Fixture\FirstClassCallables;

final class SomeCalledMethod
{
    public function callMe($number)
    {
    }
}
